// barcodereader.cpp - Barcode read from DataMatrixReader library
// (C) 2017. IntBuSoft ltd. 
#include <stdio.h>
#include "..\..\include\DataMatrixReader.h"
#include "opencv2/highgui/highgui_c.h"
#include "stdlib.h" 
#include <time.h>  

void DrawIm(IplImage* Image24, DM_RESULT* dm_result, int all_result)
{
	CvFont font;
	float aa = 0.001f*Image24->width;
	cvInitFont(&font, CV_FONT_HERSHEY_SIMPLEX, aa,
		aa, 0, 1, 8);
	for (int i = 0; i < all_result; i++)
	{
		for (int j = 0; j < 4; j++)
			cvLine(Image24, dm_result[i].Points[j], dm_result[i].Points[(j + 1) % 4], CV_RGB(255, 0, 0), 2);

		cvPutText(Image24, dm_result[i].result, dm_result[i].Points[0], &font, CV_RGB(0, 255, 0));
		printf("%s\n", dm_result[i].result);
	}
}

int main(int argc, char** argv)
{
	bool help = true;

	char* infile;
	char* outfile;

	char mode = 0; // 1 - file, 2 -camera

	if (argc == 4 && strcmp(argv[1], "-im") == 0)
	{
		mode = 1;
		help = false;
		infile = argv[2];
		outfile = argv[3];
	}

	if (argc == 3 && strcmp(argv[1], "-vc") == 0)
	{
		mode = 2;
		help = false;		
		outfile = argv[2];
	}

	if (help)
	{
		printf("barcodereader help. Format:\n");
		printf("%s -im <graphic filename> <graphic outfilename>\n",argv[0]);
		printf(" - Read graphic file\n");
		printf("%s -vc <graphic outfilename>\n", argv[0]);
		printf(" - Read web cam\n");
		return 0;
	}

	if (mode == 1)
	{
		IplImage* Image = cvLoadImage(infile, CV_LOAD_IMAGE_GRAYSCALE);

		int all_result = 10;
		DM_RESULT dm_result[10];		
		float fTimeStart = clock() / (float)CLOCKS_PER_SEC;
		//int res = DataMatrixCodeReader(Image, dm_result, &all_result, 31, DM_ATHRESHOLD);
		int res = DataMatrixCodeReader(Image, dm_result, &all_result, 128, DM_THRESHOLD);
		float fTimeStop = clock() / (float)CLOCKS_PER_SEC;
		printf("Time: %.3fs\n", fTimeStop - fTimeStart);

		if (res == 0)
		{
			IplImage* Image24 = cvLoadImage(infile, 1);
			DrawIm(Image24, dm_result, all_result);			
			cvSaveImage(outfile, Image24);
			cvReleaseImage(&Image24);
		}
		cvReleaseImage(&Image);
	}

	if (mode == 2)
	{
		CvCapture* capture = cvCaptureFromCAM(0);		
		IplImage* grayFrame = 0;

		for (;;)
		{
			IplImage* frame = 0;
			frame = cvQueryFrame(capture);
			if (!frame)
				break;
			if (!grayFrame)
				grayFrame = cvCreateImage(cvGetSize(frame), 8, 1);
			cvCvtColor(frame, grayFrame, CV_BGR2GRAY);
			
			int all_result = 10;
			DM_RESULT dm_result[10];
			float fTimeStart = clock() / (float)CLOCKS_PER_SEC;
			//int res = DataMatrixCodeReader(Image, dm_result, &all_result, 31, DM_ATHRESHOLD);
			int res = DataMatrixCodeReader(grayFrame, dm_result, &all_result, 128, DM_THRESHOLD);
			float fTimeStop = clock() / (float)CLOCKS_PER_SEC;
			printf("Time: %.3fs\n", fTimeStop - fTimeStart);
			
			DrawIm(frame, dm_result, all_result);
			cvShowImage("frame", frame);
			cvSaveImage(outfile, frame);

			int c = cvWaitKey(20);
			if (c == 32)
				c = cvWaitKey(0); // pause
			if (c == 27) break;
		}
		cvReleaseCapture(&capture);
	}
};